<?php

namespace IMATHUZH\Qfq\Core\Report;

use IMATHUZH\Qfq\Core\Typo3\FrontendUserRepository;
use IMATHUZH\Qfq\Core\Typo3\QfqAuthenticationService;

class AuthLink extends Link
{
    protected string $username;
    protected string $userdata;
    protected string $features;


    public function renderLink($str, $strDefault = '')
    {
        // Syntax:
        //    <username>|<user data>|<features>|<rendering>
        // where rendering is either a URL or a string for a column of type _link

        $parts = explode('|', $str, 4);
        $this->username = trim($parts[0]);
        $this->userdata = trim($parts[1] ?? '');
        $this->features = trim($parts[2] ?? '');
        $linkArgs = trim($parts[3] ?? '') ?: $this->getDefaultLinkArgument();
        $content = parent::renderLink($linkArgs, '');

        // In case no redirection took place and a link will be present,
        // check the account status and populate {{VAR_AUTH_RESULT:V}}
        $this->store->setVar(
            VAR_AUTH_RESULT,
            $this->getAccountStatus($this->username),
            STORE_VAR,
            true
        );

        return $content;
    }

    private function getDefaultLinkArgument()
    {
        return 'u:' . $this->getCurrentUrl();
    }

    private function getCurrentUrl()
    {
        return $_SERVER['REQUEST_URI'];
    }

    protected function initVars()
    {
        $vars = parent::initVars();
        // Default rendering mode depends on the arguments
        // and it will be computed later
        $vars[NAME_RENDER] = -1;
        $vars[FINAL_HREF] = $this->getCurrentUrl();
        return $vars;
    }

    protected function getModeRender(array $vars, array $tokenGiven)
    {
        // Update the default rendering mode
        if ($vars[NAME_RENDER] === -1) {
            $vars[NAME_RENDER] = $vars[FINAL_CONTENT]
                ? RENDER_MODE_0_LINK      // text, button, image, etc.
                : RENDER_MODE_9_REDIRECT; // only url
        }
        return parent::getModeRender($vars, $tokenGiven);
    }

    protected function doHref(array &$vars, array $tokenGiven)
    {
        $href = parent::doHref($vars, $tokenGiven);
        // create entries in the sip store for user data and authentication features
        $query = QfqAuthenticationService::generateUrlParameters(
            $this->sip,
            $this->username,
            $this->userdata,
            $this->features
        );
        // add parameters to the url
        return $href . (strpos($href, '?') === false ? '?' : '&')
                     . http_build_query($query);
    }

    protected function getAccountStatus(string $username): string
    {
        $status = FrontendUserRepository::getInstance()->getStatus($username);
        return (empty($status) ? AUTH_STATUS_NOT_FOUND : '')
            ?: ($status['locked'] ? AUTH_STATUS_LOCKED : '')
            ?: ($status['expired'] ? AUTH_STATUS_EXPIRED : '')
            ?: ($status['inactive'] ? AUTH_STATUS_INACTIVE : '')
            ?: AUTH_STATUS_OK;
    }
}